#
#  @file   PythonLoggerForParaxip.py
#  @author Stephan Goulet
#  date    November 27, 2008
#  
#  Description: This class is a customized version of the generic python logger.
#               It allows logging to both the python and the paraxip logging
#               subsystem.
#
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#
import logging
import types
import netborderPyInfra as pxip



# This class can be used in two ways:
# 
# Option 1: Use the class as it is with no change, by simply calling :
# loggging.setLoggerClass(PythonLoggerForParaxip). 
# This will result in all logs being sent to both the python logger and the
# Paraxip logger.
# 
# Option 2: Derive your own class from PythonLoggerForParaxip and implement
# a constructor that will set the Paraxip prefix to each logger. This 
# will ensure that all log from this class will be generated by the same 
# "root" logger. See PythonLoggerForParaxip.t.py for an example
# 

class PythonLoggerForParaxip(logging.Logger):
  def __init__ (self, in_strLoggerName, level=logging.NOTSET, 
                in_strPxipPrefix=None, in_bEnablePythonLog = True ):    
    if in_strPxipPrefix is None:
      self._logger = pxip.Logger(in_strLoggerName)    
    else: 
      # Prepend the prefix (e.g. "netborder.turboGears") to the logger name
      # Used when constructor is called by a derived class
      strLoggerName = in_strPxipPrefix + "."
      strLoggerName += in_strLoggerName
      self._logger = pxip.Logger(strLoggerName)    
      
    self._enablePythonLog = in_bEnablePythonLog      
    logging.Logger.__init__(self,in_strLoggerName, level)

  # This will set the paraxip log level. To set the python log level, 
  # simply use setLevel()
  def setParaxipLevel(self, level):
    pxipLevel = self.getPxipLevel(level)
    self._logger.setLogLevel(pxipLevel)    

  def critical(self, msg, *args, **kwargs):
    if self._logger.isEnabledFor(pxip.LogLevel.fatal) :
      msg2 = self.getMessage(msg, args)      
      self._logger.forcedLog(pxip.LogLevel.fatal,msg2)
    if self._enablePythonLog:
      logging.Logger.critical(self, msg, *args, **kwargs)

  def exception(self, msg, *args, **kwargs):
    if self._logger.isEnabledFor(pxip.LogLevel.error) :
      msg2 = self.getMessage(msg, args)      
      self._logger.forcedLog(pxip.LogLevel.error,msg2)
    if self._enablePythonLog:
      logging.Logger.exception(self, msg, *args, **kwargs)

  def error(self, msg, *args, **kwargs):
    if self._logger.isEnabledFor(pxip.LogLevel.error) :
      msg2 = self.getMessage(msg, args)      
      self._logger.forcedLog(pxip.LogLevel.error,msg2)
    if self._enablePythonLog:
      logging.Logger.error(self, msg, *args, **kwargs)

  def warning(self, msg, *args, **kwargs):
    if self._logger.isEnabledFor(pxip.LogLevel.warn) :
      msg2 = self.getMessage(msg, args)      
      self._logger.forcedLog(pxip.LogLevel.warn,msg2)
    if self._enablePythonLog:
      logging.Logger.warning(self, msg, *args, **kwargs)

  def info(self, msg, *args, **kwargs):
    if self._logger.isEnabledFor(pxip.LogLevel.info) :            
      msg2 = self.getMessage(msg, args)      
      self._logger.forcedLog(pxip.LogLevel.info,msg2)     
    if self._enablePythonLog:
      logging.Logger.info(self, msg, *args, **kwargs)

  def debug(self, msg, *args, **kwargs):
    if self._logger.isEnabledFor(pxip.LogLevel.debug) :
      msg2 = self.getMessage(msg, args)      
      self._logger.forcedLog(pxip.LogLevel.debug,msg2)
    if self._enablePythonLog:
      logging.Logger.debug(self, msg, *args, **kwargs)

  def getPxipLevel (self, in_pythonLevel):
    if in_pythonLevel == logging.CRITICAL:
      return pxip.LogLevel.fatal
    if in_pythonLevel == logging.ERROR:
      return pxip.LogLevel.error
    if in_pythonLevel == logging.WARNING or in_pythonLevel == logging.WARN:
      return pxip.LogLevel.warn
    if in_pythonLevel == logging.INFO:
      return pxip.LogLevel.info
    if in_pythonLevel == logging.DEBUG:
      return pxip.LogLevel.debug
    else:
      # Not sure what to do. Normal case when called by ctor
      return pxip.LogLevel.debug

  # TODO: Check if we could be more efficient here
  def getMessage(self, msg, args):
    """     
    Return the message for this LogRecord after merging any user-supplied
    arguments with the message.
    """
    # Paraxip logger does not support unicode, so we need to transform this 
    # string in a normal python string if it is unicode.
    # See logger LogRecord.getMessage for inspiration to this method.
     
    # Note: Don't think we need to do this translation here, but done in 
    # LogRecord.getMessage
    msg2 = str(msg)

    if args:
        msg2 = msg2 % args
    msg2 = str(msg2)
    return msg2



## !!MIRROR FILE, DO NOT MODIFY!!
