#
#  @file   Converter.py
#  @author Jean-Philippe Boily, jr. eng.
#  @date   02/14/2011
#
#  Description:
#
#  Defines the converters class that can be used in the smart update process.
#
#  Copyright (C) 2011 Sangoma Technologies.
#  All Rights Reserved.
#
#  This code is Sangoma Technologies Confidential Property.
#  Use of and Access to this code is covered by a previously executed
#  Non-disclosure Agreement between Sangoma Technologies and the Recipient.
#  This code is being supplied for evaluation purposes only and is not to be
#  used for any other purpose.
#
#

__author__="Jean-Philippe Boily, jr. eng."
__date__ ="$14-Feb-2011 9:49:34 AM$"

#-----------------------------------------------------------------------------#
# imports
#-----------------------------------------------------------------------------#
import os
import os.path
import netborderPyXML
import shutil

from paraxip.infra.PyJob import Job
from paraxip.infra.PyLoggingFunctions import *
from netborderPyInfra import PyGlobalConfig as globalCfg
from netborderPyInfra import StringVector
from netborderPyInfra import ParameterTrie

from TempDirManager import *
from XsltProcWrapper import *

#-----------------------------------------------------------------------------#
# global variables
#-----------------------------------------------------------------------------#
fileScopeLogger  = Logger("netborder.smartUpdate.Converter")
versionConverterLogger = Logger("netborder.smartUpdate.VersionConverter")
propertiesLogger = Logger("netborder.smartUpdate.Converter.PropertiesConverter")
xmlLogger        = Logger("netborder.smartUpdate.Converter.XmlConverter")
validationLogger = Logger("netborder.smartUpdate.Converter.ValidationConverter")
copyLogger       = Logger("netborder.smartUpdate.Converter.CopyConverter")
licenseLogger    = Logger("netborder.smartUpdate.Converter.LicenseValidator")
gwLicenseLogger  = Logger("netborder.smartUpdate.Converter.GwLicenseValidator")

validationFilesDir = \
  globalCfg.getParameter("netborder.smartUpdate.ValidationFiles.Directory")
conversionFilesDir = \
  globalCfg.getParameter("netborder.smartUpdate.XslScripts.Directory")

xsltProc = XsltProcWrapper()

    
#-----------------------------------------------------------------------------#
# Class Converter
#-----------------------------------------------------------------------------#
class Converter(Job):
  """
  Converter class: base class for all the converters, class takes the following
  arguments:

    inputFilename: the file to convert.
    outputFilename: the name to which the input file will be saved after the
                    conversion
    xsltScript: the xslt script to apply to the input file
    inputValidationFile: the file used to validate the input file
       
  """
  def __init__(self, tempDirManager, group, inputFilename, outputFilename, 
               jobStatus=None, xsltScript=None, inputValidationFile=None,
               eraseInputFile=True):
    super(Converter, self).__init__()
    self.tempDirManager = tempDirManager
    self.group = group
    self.inputFilename = inputFilename
    self.outputFilename = outputFilename
    self.jobStatus = jobStatus
    self.xsltScript = xsltScript
    self.inputValidationFile = inputValidationFile
    self.validationFilesDir = validationFilesDir
    self.conversionFilesDir = conversionFilesDir

    #If eraseInputFile = false (as a string) set it to False (bool)
    if isinstance(eraseInputFile, basestring):
      if eraseInputFile.lower() == 'false':
        self.eraseInputFile = False
      else:
        self.eraseInputFile = True
    else:
      self.eraseInputFile = eraseInputFile

    if self.tempDirManager is None:
      log_error(fileScopeLogger, "failed to set TempDirManager instance")

  def run(self):
    """Abstract method should not be called"""
    raise NotImplementedError('Converter.run should not be called.'\
                              'Abstract class!')
    return False

  def getValidationFilePath(self, validationFileName):
    """Return the path to a validation file. Start by looking into the
    validationFilesDir and if not look into patch data directory."""

    log_debug(fileScopeLogger, "Looking for validation file %s in %s" % 
        (validationFileName, self.validationFilesDir))

    validationFilePath = os.path.join(self.validationFilesDir, 
                                      validationFileName)

    if not os.path.exists(validationFilePath):

      patchDataDirPath = self.tempDirManager.getPatchDataDirPath()
      log_debug(fileScopeLogger, "Looking for validation file %s in " \
                "patch data directory %s" % 
                (validationFileName, patchDataDirPath)
      )
      validationFilePath = os.path.join(patchDataDirPath, validationFileName)

    if os.path.exists(validationFilePath):
      log_debug(fileScopeLogger, "Found validation file at %s" %
                validationFilePath)
      return validationFilePath
    else:
      log_error(fileScopeLogger, "Can't find validation file named %s" %
               validationFileName)
      return None

  def getConversionFilePath(self, conversionFileName):
    """Return the path to a conversion file. Start by looking into the
    conversionFilesDir and if not look into the patch data directory."""

    conversionFilePath = os.path.join(self.conversionFilesDir, 
                                      conversionFileName)
    if not os.path.exists(conversionFilePath):

      patchDataDirPath = self.tempDirManager.getPatchDataDirPath()
      log_debug(fileScopeLogger, "Looking for conversion file %s in " \
                "patch data directory %s" % 
                (conversionFileName, patchDataDirPath)
      )
      conversionFilePath = os.path.join(patchDataDirPath, conversionFileName)

    if os.path.exists(conversionFilePath):
      log_debug(fileScopeLogger, "Found conversion file at %s" %
                conversionFilePath)
      return conversionFilePath
    else:
      log_error(fileScopeLogger, "Can't find conversion file named %s" %
               conversionFileName)
      return None


  def runXsltProc(self, xsltScript, xmlInputFile, outputFile, opts=None):
    # configure xsltProc to run the inputFile modification script
    xsltProc.configure(xsltScript, xmlInputFile, outputFile, xsltProcOptions=opts)

    (rc, msg, warningList) = xsltProc.call()
    if not rc == 0:
      log_error(
        fileScopeLogger,
        "xsltProc failed to process %s file. cause=%s." % (xmlInputFile, rc))
      return False

    if not warningList == []:
      for warnMsg in warningList:
        self.jobStatus.addWarning(warnMsg)

    return True

  def validateXmlFile(self,inputFilePath,validationFile):
    validator = netborderPyXML.XmlRelaxngValidator()
    strSchemaFile = self.getValidationFilePath(validationFile)
    if strSchemaFile is None:
      return False

    log_debug(fileScopeLogger,"xml file schema: %s" % strSchemaFile)
    validator.configure(strSchemaFile)
    if not validator.validateXmlFile(inputFilePath):
      log_error(
        fileScopeLogger, "Invalid file %s" % inputFilePath
      )
      return False

    return True

  def validatePropertiesFile(self,inout_paramTrie,inputFilePath,validationFile):

    # User may not specify a validation file
    paramDB = None
    if len(validationFile) != 0:
      validationFilePath = self.getValidationFilePath(validationFile)
      if validationFilePath:
        paramDB = ParameterInfoDB.loadFromXmlFile(validationFilePath)
    
    inputFiles = StringVector()
    inputFiles.extend(inputFilePath.split(','))

    if not inout_paramTrie.load(inputFiles, paramDB):
      log_error( fileScopeLogger,
                 "Invalid input file(s) %s. Stopping process." % inputFilePath )
      return False

    return True

  def savePropFileWithDocAndDefaults(self, inout_paramTrie, inputFilePath,
                                     outputFilePath, validationFile):
                                     
    # User may not specify a validation file
    paramDB = None
    if len(validationFile) != 0:
      validationFilePath = self.getValidationFilePath(validationFile)
      log_debug(fileScopeLogger, "openning parameter DB file %s" % validationFilePath )
      if validationFilePath:
        paramDB = ParameterInfoDB.loadFromXmlFile(validationFilePath)
        if paramDB is None:
          log_error(fileScopeLogger, "failed to load paramter DB file: %s" % validationFilePath )
          return False


    inputFiles = StringVector()
    inputFiles.extend(inputFilePath.split(','))

    if not inout_paramTrie.load(inputFiles, paramDB):
      log_error( fileScopeLogger,
                 "Invalid input files %s. Stopping process." % inputFilePathb )
      return False

    if paramDB is None:
      log_debug(fileScopeLogger, "Writing properties files '%s'" % outputFilePath)
      if not inout_paramTrie.saveAsPropertiesFile(outputFilePath):
        log_error( fileScopeLogger,
                    "Failed to write properties file '%s'" % outputFilePath )
        return False

    else:
      log_debug(fileScopeLogger, "Writing properties files '%s' with"\
                "default parameters and documentation" % outputFilePath)
      paramDB.addDefaultsToConfig(inout_paramTrie, False)
      if not inout_paramTrie.saveAsPropertiesFile(outputFilePath,
                                                  paramDB, True):
        log_error( fileScopeLogger,
                    "Failed to write properties file '%s'" % outputFilePath )
        return False


    return True

  def copyFile(self, inputFilePath, outputFilePath):
    log_debug(fileScopeLogger,"input path: %s" % inputFilePath)
    log_debug(fileScopeLogger,"output path: %s" % outputFilePath)
    try:
      shutil.copy(inputFilePath,outputFilePath)
    except IOError, e:
      log_error(
        fileScopeLogger,
        "Unable to copy file %s to %s.error=%s" \
        % (inputFilePath, outputFilePath, e)
      )
      return False

    return True

  def copyDir(self, inputDirPath, outputDirPath):
    """Copy all files and subdirectories of inputDirPath to outputDirPath"""

    srcFileList = list()
    for currentDir, subDir, files in os.walk(inputDirPath):
      for currentFile in files:
        srcFileList.append(os.path.join(currentDir, currentFile))

    log_debug(fileScopeLogger, "Copying directory tree '%s' to '%s'" 
                % (inputDirPath, outputDirPath))

    for srcFile in srcFileList:
      dstFile = srcFile.replace(inputDirPath, '')
      dstFilePath = os.path.join(outputDirPath, dstFile)
      dstDirPath = os.path.dirname(dstFilePath)

      if not os.path.exists(dstDirPath):
        #Sub directory must be created before copying the file.
        log_debug(fileScopeLogger, "Creating subdirectories path '%s'" %
                  dstDirPath)
        try:
          os.makedirs(dstDirPath)
        except OSError, e:
          log_error(fileScopeLogger, "Unable to create subdirectories path "\
                    "'%s'. Error=%s" % (dstDirPath, e))
          return False
      
      if not self.copyFile(srcFile, dstFilePath):
        return False

    return True


  def removeFile(self, filePath):
    log_debug(fileScopeLogger,"removing file %s" % filePath)
    try:
      os.remove(filePath)
    except IOError, e:
      log_error(
        fileScopeLogger,"failed to remove file %s. error=%s" % (filePath,e)
      )
      return False
    return True

  def removeDir(self, dirPath):
    """Delete a directory tree"""
    log_debug(fileScopeLogger, "Deleting directory '%s'" % dirPath)
    try:
      shutil.rmtree(dirPath)
    except (OSError, IOError), e:
      log_error(fileScopeLogger, "Unable to remove directory '%s'. Error=%s" %
                (dirPath, e))
      return False
    return True


#-----------------------------------------------------------------------------#
# Class VersionConverter
#-----------------------------------------------------------------------------#
class VersionConverter(Converter):
  """
  Converter use to convert a versionize file, this converter add the
  following arguments:

    srcVersion: the current version of the input file
    dstVersion: the version of the file after the conversion
  """

  def __init__(self, tempDirManager, srcVersion, dstVersion, group,
                inputFilename, outputFilename, jobStatus=None, xsltScript=None,
                inputValidationFile=None, eraseInputFile=True):
    t = TraceScope(versionConverterLogger, "VersionConverter.__init__")
    super(VersionConverter, self).__init__(tempDirManager, group, inputFilename,
                                           outputFilename, jobStatus, 
                                           xsltScript, inputValidationFile,
                                           eraseInputFile)
    self.srcVersion = srcVersion
    self.dstVersion = dstVersion

  def run(self):
    """Abstract method should not be called"""
    t = TraceScope(versionConverterLogger, "VersionConverter.run")
    raise NotImplementedError('VersionConverter.run should not be called.'\
                              'Abstract class!')
    return False

#-----------------------------------------------------------------------------#

#-----------------------------------------------------------------------------#
# Class PropertiesConverter
#-----------------------------------------------------------------------------#
class PropertiesConverter(VersionConverter):
  """
  PropertiesConverter class:

  Converts an application properties file from one version to another by
  applying a xslt modification script."""
  def __init__(self, tempDirManager, srcVersion, dstVersion, group, 
                inputFilename, outputFilename, xsltScript,
                inputValidationFile, jobStatus=None, eraseInputFile=True):
    super(PropertiesConverter, self).\
      __init__(tempDirManager, srcVersion, dstVersion, group, inputFilename,
                outputFilename, jobStatus, xsltScript, inputValidationFile,
               eraseInputFile)

  def run(self):
    log_info( 
      propertiesLogger,
      "Updating file(s) %s from version %s to version %s" \
      % (self.inputFilename, self.srcVersion, self.dstVersion)
    )

    srcDir = self.tempDirManager.getSrcDir()
    log_debug(propertiesLogger,"src dir: %s" % srcDir)
    
    inputFilePath = ''
    inputFiles = self.inputFilename.split(',')
    log_debug(validationLogger, "inputFiles=%s" % inputFiles )
    for file in inputFiles:
      log_debug(validationLogger, "file=%s" % file )
      filePath = os.path.join(srcDir, file)
      log_debug(validationLogger, "src file: %s" % filePath)
      
      #  Don't run convertion of the one input file does not exists
      if not os.path.exists(filePath):
        log_info(validationLogger,"File %s does not exists" % filePath)
        return True
      
      if len(inputFilePath) == 0:
        inputFilePath = filePath
      else:
        inputFilePath += ',' + filePath
        
      (inFilename,extension) = os.path.splitext(file)    
    

    dstDir = self.tempDirManager.getDstDir()
    log_debug(propertiesLogger,"dst dir: %s" % dstDir)
    outputFilePath = os.path.join(dstDir, self.outputFilename)

    (outFilename,extension) = os.path.splitext(self.outputFilename)
    xmlPropertiesInFileName = "temp-input-file-xml"
    xmlPropertiesOutFileName = outFilename + ".xml"
    xmlPropertiesInPath = os.path.join(srcDir, xmlPropertiesInFileName)
    xmlPropertiesOutPath = os.path.join(dstDir, xmlPropertiesOutFileName)

    conversionFile = self.getConversionFilePath(self.xsltScript)
    if conversionFile is None:
      return False

    # load the input file to validate it and save it as xml
    paramTrie = ParameterTrie()
    if not self.validatePropertiesFile(paramTrie, inputFilePath,
                                        self.inputValidationFile):
      log_error(
        propertiesLogger,
        "Invalid input file(s) %s. Stopping process." % self.inputFilename
      )
      return False

    # Why this is required to save in Xml ? The function saveAsXmlFile shall support None paramDB
    paramDBFile = None
    if len(self.inputValidationFile) != 0:
      paramDBFile = self.getValidationFilePath(self.inputValidationFile)
    if paramDBFile is not None: 
      log_debug( propertiesLogger,
                 "Saving properties file %s as xml with documentation" % xmlPropertiesInPath )
      paramDB = ParameterInfoDB.loadFromXmlFile(paramDBFile)
      if not paramTrie.saveAsXmlFile(xmlPropertiesInPath, paramDB):
        log_error( propertiesLogger,
                  "Unable to save properties file %s as xml." % inputFilePath )
        return False
    else:   

      log_debug( propertiesLogger,
                 "Saving properties file %s as xml without documentation" % xmlPropertiesInPath )
      if not paramTrie.saveAsXmlFileNoDB(xmlPropertiesInPath):
        log_error( propertiesLogger,
                  "Unable to save properties file %s as xml." % inputFilePath )
        return False

    # configure xsltProc to run the inputFile modification script
    # !+! jpboily : Bug 6653 - Passing versions to the properties conversion
    # xsl script to give them more flexibility. Will use the dstVersion value
    # in the final job to make sure the parameters having for value the gw
    # version will be set to the right version.
    xsltProcOpts = [
      "--stringparam srcVersion %s" % self.srcVersion,
      "--stringparam dstVersion %s" % self.dstVersion
    ]
    if not self.runXsltProc(conversionFile, xmlPropertiesInPath,
                            xmlPropertiesOutPath, xsltProcOpts):
      log_error(
        propertiesLogger,
        "Failed to modify properties file %s" % xmlPropertiesInFileName
      )
      return False

    if not self.removeFile(xmlPropertiesInPath):
      log_error(
        propertiesLogger,
        "failed to remove input xml properties file %s" % xmlPropertiesInPath
      )

    xmlToPropScript = self.getConversionFilePath('xmlToProperties.xsl')
    if xmlToPropScript is None:
      return False
    
    # configure xsltProc to run the xmlToProperties script
    if not self.runXsltProc(xmlToPropScript, xmlPropertiesOutPath,
                            outputFilePath):
      log_error( propertiesLogger,
                 "Failed to convert back %s to properties format." \
                  % xmlPropertiesInFileName)
      return False

    if not self.removeFile(xmlPropertiesOutPath):
      log_error( propertiesLogger,
                "failed to remove output xml properties file %s" \
                % xmlPropertiesOutPath )
     
    if self.eraseInputFile is True:
      inputFilesList = inputFilePath.split(',')
      for file in inputFilesList:
        self.removeFile(file)
      
    return True

#-----------------------------------------------------------------------------#

#-----------------------------------------------------------------------------#
# Class XmlConverter
#-----------------------------------------------------------------------------#
class XmlConverter(VersionConverter):
  """
  XmlConverter class:

  Converts a xml properties file from one version to another by
  applying a xslt modification script."""
  def __init__(self, tempDirManager, srcVersion, dstVersion, group, 
              inputFilename, outputFilename, xsltScript, 
              inputValidationFile, jobStatus=None, eraseInputFile=True):
    super(XmlConverter, self).\
      __init__(tempDirManager, srcVersion, dstVersion, group, inputFilename,
                outputFilename, jobStatus, xsltScript, inputValidationFile,
               eraseInputFile)

  def run(self):
    log_info(
      xmlLogger,
      "Updating file %s from version %s to version %s" \
      % (self.inputFilename, self.srcVersion, self.dstVersion)
    )

    srcDir = self.tempDirManager.getSrcDir()
    inputFilePath = os.path.join(srcDir, self.inputFilename)
    log_debug(xmlLogger,"src file: %s" % inputFilePath)

    # Do not try to do the conversion if the file is not there.
    if not os.path.exists(inputFilePath):
      log_info(xmlLogger,"File %s does not exists" % inputFilePath)
      return True

    dstDir = self.tempDirManager.getDstDir()
    outputFilePath = os.path.join(dstDir, self.outputFilename)
    log_debug(xmlLogger,"dst file: %s" % outputFilePath)

    conversionFile = self.getConversionFilePath(self.xsltScript)
    if conversionFile is None:
      return False

    log_debug(xmlLogger,"conversion file: %s" % conversionFile)

    if not self.validateXmlFile(inputFilePath, self.inputValidationFile):
      log_error(
        xmlLogger,
        "Failed to update input file %s. File is invalid" % self.inputFilename
      )
      return False

    # configure xsltProc to run the inputFile modification script
    if not self.runXsltProc(conversionFile, inputFilePath, outputFilePath):
      log_error(
        propertiesLogger,
        "Failed to modify xml file %s" % self.inputFilename
      )
      return False

    if self.eraseInputFile is True:
      return self.removeFile(inputFilePath)
    else:
      return True

#-----------------------------------------------------------------------------#

#-----------------------------------------------------------------------------#
# Class ValidationConverter
#-----------------------------------------------------------------------------#
class ValidationConverter(VersionConverter):
  """
  ValidationConverter class:

  Validates the input file and copies it to the destination directory."""
  def __init__(self, tempDirManager, srcVersion, dstVersion, group, 
                inputFilename, outputFilename, jobStatus=None, xsltScript=None,
                inputValidationFile=None, eraseInputFile=True):
    super(ValidationConverter, self).\
      __init__(tempDirManager, srcVersion, dstVersion, group, inputFilename,
                outputFilename, jobStatus, xsltScript, inputValidationFile,
               eraseInputFile)

  def run(self):
    log_info(  validationLogger,
               "Validating files [%s] at version %s" % (self.inputFilename, self.srcVersion) )

    srcDir = self.tempDirManager.getSrcDir()

    inputFilePath = ''
    inputFiles = self.inputFilename.split(',')
    for file in inputFiles:
      filePath = os.path.join(srcDir, file)
      log_debug(validationLogger, "src file: %s" % filePath)
      
      #  Don't run convertion of the one input file does not exists
      if not os.path.exists(filePath):
        log_info(validationLogger,"File %s does not exists" % filePath)
        return True
      
      if len(inputFilePath) == 0:
        inputFilePath = filePath
      else:
        inputFilePath += ',' + filePath
        
      (filename,extension) = os.path.splitext(file)    
      
    dstDir = self.tempDirManager.getDstDir()
    outputFilePath = os.path.join(dstDir, self.outputFilename)
    log_debug(validationLogger, "dst file: %s" % outputFilePath)

    if extension == ".xml":
      if not self.validateXmlFile(inputFilePath, self.inputValidationFile):
        log_error(
          validationLogger,
          "Failed to update input file %s. File is invalid" % self.inputFilename
        )
        return False

      if not self.copyFile(inputFilePath,outputFilePath):
        return False

    elif extension == ".properties":
      log_info( validationLogger, 
                "saving properties file %s with documentation" % outputFilePath )
      paramTrie = ParameterTrie()
      if not self.savePropFileWithDocAndDefaults(paramTrie,
                                                 inputFilePath,
                                                 outputFilePath,
                                                 self.inputValidationFile):
        return False

    if self.eraseInputFile is True:
      return self.removeFile(inputFilePath)
    else:
      return True


#-----------------------------------------------------------------------------#
# Class CopyConverter
#-----------------------------------------------------------------------------#
class CopyConverter(VersionConverter):
  """
  CopyConverter class:

  Copies the input file to the destination directory."""
  def __init__(self, tempDirManager, srcVersion, dstVersion, group,
                inputFilename, outputFilename, jobStatus=None, xsltScript=None,
                inputValidationFile=None, eraseInputFile=True):
    super(CopyConverter, self).\
      __init__(tempDirManager, srcVersion, dstVersion, group, inputFilename,
                outputFilename, jobStatus, xsltScript, inputValidationFile,
              eraseInputFile)

  def run(self):
    srcDir = self.tempDirManager.getSrcDir()
    log_debug(copyLogger, "src dir: %s" % srcDir)
    dstDir = self.tempDirManager.getDstDir()
    log_debug(copyLogger, "dst dir: %s" % dstDir)

    inputFilePath = os.path.join(srcDir, self.inputFilename)
    outputFilePath = os.path.join(dstDir, self.outputFilename)

    elementToCopy = 'file'
    copyDir = False
    if os.path.isdir(inputFilePath):
      elementToCopy = 'directory'
      copyDir = True

    log_info(
      copyLogger,
      "Copying %s %s"  % (elementToCopy, self.inputFilename)
    )

    # Do not try to do the conversion if the file is not there.
    if not os.path.exists(inputFilePath):
      log_info(copyLogger,"File %s does not exists" % inputFilePath)
      return True

    if copyDir:
      if not self.copyDir(inputFilePath, outputFilePath):
        return False

      if self.eraseInputFile is True:
        return self.removeDir(inputFilePath)
      else:
        return True

    else:
      if not self.copyFile(inputFilePath,outputFilePath):
        return False

      if self.eraseInputFile is True:
        return self.removeFile(inputFilePath)
      else:
        return True


#-----------------------------------------------------------------------------#
# Class LicenseValidator
#-----------------------------------------------------------------------------#
class LicenseValidator(VersionConverter):
  """Validate a license.txt file to make sure that it can
     be used. If not warn the user. """

  def __init__(self, tempDirManager, srcVersion, dstVersion, group,
                inputFilename, outputFilename, jobStatus=None, xsltScript=None,
                inputValidationFile=None, eraseInputFile=True):

    t = TraceScope(licenseLogger, "LicenseValidator.__init__")
    super(LicenseValidator, self).\
      __init__(tempDirManager, srcVersion, dstVersion, group, inputFilename,
                outputFilename, jobStatus, xsltScript, inputValidationFile,
               eraseInputFile)
    self.inputLicensePath = None
    self.outputLicensePath = None
    self.licenseParamsDict = None

  def updateAttributes(self):
    """Set object attributes based on the status of the TempDirManager"""
    t = TraceScope(licenseLogger, "LicenseValidator.updateAttributes")

    srcDir = self.tempDirManager.getSrcDir()
    log_debug(licenseLogger, "src dir: %s" % srcDir)
    dstDir = self.tempDirManager.getDstDir()
    log_debug(licenseLogger, "dst dir: %s" % dstDir)

    self.inputLicensePath = os.path.join(srcDir, self.inputFilename)
    log_debug(
      licenseLogger,
      "Input license file path: %s" % self.inputLicensePath
    )
    if not os.path.exists(self.inputLicensePath):
      log_info(licenseLogger,"File %s does not exists" % self.inputLicensePath)
      return False

    self.outputLicensePath = os.path.join(dstDir, self.outputFilename)
    log_debug(
      licenseLogger,
      "Output license file path: %s" % self.outputLicensePath
    )

    self.licenseParamsDict = self.getLicenseParams(self.inputLicensePath)

    return True
    

  def run(self):
    """Validate input license file, if it's valid copy it to output path."""
    t = TraceScope(licenseLogger, "LicenseValidator.run")
    majorVersionParamName = 'netborder.license.productMajorVersion'

    log_info(
      licenseLogger,
      "Validating license file %s with version %s of the application" %
      (self.inputFilename, self.dstVersion)
    )

    #If False this mean that there is not license file to process.
    if not self.updateAttributes():
      return True


    if majorVersionParamName not in self.licenseParamsDict:
      log_error(
        licenseLogger,
        "Failed to retrieve parameter '%s' in license '%s'" % 
        (majorVersionParamName, self.inputFilename)
      )
      return False

    inLicenseMajorVersion = \
        self.licenseParamsDict[majorVersionParamName]

    log_debug(
      licenseLogger,
      "License major version = %s" % inLicenseMajorVersion
    )

    #Check if the license is not version restricted.
    #If license version is either 99 or 999, copy the license from the archive.
    #If not the license is left as it is (should be the default one...)
    if inLicenseMajorVersion == '99' or inLicenseMajorVersion == '999':
      log_debug(
        licenseLogger,
        "License '%s' is not version restricted" % self.inputLicensePath
      )
      #Copy license and associated .sig
      if self.copyFile(self.inputLicensePath, self.outputLicensePath):
        inSigFile = self.inputLicensePath + ".sig"
        outSigFile = self.outputLicensePath + ".sig"
        return self.copyFile(inSigFile, outSigFile)
      else:
        log_error(
          licenseLogger,
          "failed to copy license file %s" % self.inputLicensePath
        )
        return False
    else:

      log_debug(
        licenseLogger,
        "License '%s' is version restricted" % self.inputLicensePath
      )

      self.jobStatus.addWarning(
        "The license file %s inside the imported archive is not valid for "\
        "version %s of the application. Default license will be used, "\
        "contact support if you require more channels." % 
        (self.inputFilename, self.dstVersion)
      )
      return True


  def getLicenseParams(self, licensePath):
    """Retrieve input license file parameters"""
    t = TraceScope(fsLogger, "LicenseValidator.getLicenseParams")
    log_debug(
      licenseLogger,
      "Retrieving license parameters of %s" % licensePath
    )
    paramsDict = dict()
    try:
      licenseFile = open(licensePath)
      for line in licenseFile:
        #Ignore comments and empty lines.
        if line.startswith(('#', '\n', '\r')):
          continue
        #This is just to make sure that we do not process empty line.
        elif not line.strip():
          continue

        log_debug(
          licenseLogger,
          "Parsing line : %s", line
        )
        param = line.split('=')

        #Return error if param list length is not 2.
        if len(param) != 2:
          log_error(
            licenseLogger,
            "Error while processing license file %s. License format seem "\
            "invalid." % licensePath
          )
          return None

        paramName = param[0]
        #Remove trailing \r \n
        paramValue = param[1].rstrip()
        paramsDict[paramName] = paramValue
        log_debug(
          licenseLogger,
          "Found license parameter '%s' with value of '%s'" %
          (paramName, paramValue)
        )
      licenseFile.close()
    except IOError, e:
      log_error(
        licenseLogger,
        "Failed to open license file '%s'. Error=%s" % 
        (licensePath, e)
      )
      return None

    return paramsDict



#-----------------------------------------------------------------------------#
# Class GwLicenseValidator
#-----------------------------------------------------------------------------#
class GwLicenseValidator(LicenseValidator):
  """Validate a license.txt file specific to the gateway and make sure 
     that it can be used. If not warn the user. """

  def __init__(self, tempDirManager, srcVersion, dstVersion, group,
                inputFilename, outputFilename, jobStatus=None, xsltScript=None,
                inputValidationFile=None, eraseInputFile=True):

    t = TraceScope(gwLicenseLogger, "GwLicenseValidator.__init__")
    super(GwLicenseValidator, self).\
      __init__(tempDirManager, srcVersion, dstVersion, group, inputFilename,
                outputFilename, jobStatus, xsltScript, inputValidationFile,
              eraseInputFile)
  
  def run(self):
    """Validate input license file, if it's valid copy it to output path.
       If runningEsp parameter is present do not modify the license (keep
       the one coming with the installer)."""
    t = TraceScope(gwLicenseLogger, "GwLicenseValidator.run")

    if not self.updateAttributes():
      return True

    if self.__isRunningEsp():
      log_debug(
        gwLicenseLogger,
        "Gateway license '%s' has parameter running Esp activated" %
        self.inputLicensePath
      )

      return True

    else:
      #Call default implementation.
      log_debug(
        gwLicenseLogger,
        "Gateway license '%s' parameter running Esp is not activated" %
        self.inputLicensePath
      )
      return super(GwLicenseValidator, self).run()
  

  def __isRunningEsp(self):
    """Find if the parameter runningOnHpEsp is set to true"""
    t = TraceScope(gwLicenseLogger, "GwLicenseValidator.__isRunningEsp")

    runningEspParamName = 'netborder.license.runningOnHpEsp'
    if self.licenseParamsDict is None:
      self.updateAttributes()

    if runningEspParamName not in self.licenseParamsDict:
      return False
    
    if self.licenseParamsDict[runningEspParamName] != 'true':
      return False
    
    return True
    
#-----------------------------------------------------------------------------#

#-----------------------------------------------------------------------------#
# Runtests
#-----------------------------------------------------------------------------#
if __name__ == "__main__":

  def runtest():
    fileScopeLogger.setLogLevel( LogLevel.debug )

    initialPath = "./test-output/test-initial-config"
    finalPath = "./test-output/test-final-config"
    toReplaceStr = "C:\Program Files\Netborder\Express\Gateway"
    replaceWithStr = globalCfg.getParameter("netborder.Installation.Directory")
    tempDirManager = TempDirManager(initialPath, finalPath, {toReplaceStr:replaceWithStr})
    tempDirManager.initialize()
    tempDirManager.initialConvert()
    tempDirManager.setTempMode()
    
    propertiesConverter = PropertiesConverter(tempDirManager, "3.0.1", "3.0.2","gw","gw_3.0.1.properties","gw_3.0.2.properties","gw_properties_3.0.1_to_3.0.2.xsl","3.0.1_gw_paramDefs.xml")
    xmlConverter = XmlConverter(tempDirManager, "3.0.0", "3.0.1","PSTN","pstn-config_3.0.0.xml","pstn-config.xml_3.0.1","pstn-config_3.0.0_to_3.0.1.xsl","3.0.0_pstn-config.rng")
    validationConverter = ValidationConverter(tempDirManager, "3.0.2", "3.0.2","gw","gw_3.0.2.properties","gw.properties",inputValidationFile="3.0.2_gw_paramDefs.xml")
    validationConverter2 = ValidationConverter(tempDirManager, "3.0.1", "3.0.1","PSTN","pstn-config.xml_3.0.1","pstn-config.xml_3.0.1",inputValidationFile="3.0.1_pstn-config.rng")
    copyConverter = CopyConverter(tempDirManager, "3.0.1", "3.0.1","oam","oamWebServer_3.0.1.properties","oamWebServer.properties")

    propertiesConverter.run()
    xmlConverter.run()
    tempDirManager.moveRemainingFiles()
    tempDirManager.swapDirectories()
    tempDirManager.setFinalMode()
    validationConverter.run()
    validationConverter2.run()
    copyConverter.run()
    
    print "Converter runtests";

  runtest()
## !!MIRROR FILE, DO NOT MODIFY!!
