"""

File: XsltProcWrapper.py
Author: Jonathan Maurice
Date: 14/02/2011
Description: Simple Xsltproc executable Wrapper.

Copyright (C) 2003-2011 Paraxip Technologies.
All Rights Reserved.

"""

from paraxip.infra.Assert import *
from paraxip.infra.PyLoggingFunctions import  *
from netborderPyInfra import PyGlobalConfig as globalConfig
from netborderPyInfra import TraceScope

import sys
import os
import re
import subprocess

fsLogger = Logger("netborder.smartUpdate.XsltProcWrapper")


class XsltProcWrapper(object):
  """Simple Xsltproc command wrapper class
     Args: 
       xslFile: path to xsl stylesheet
       xmlFile: path to xml file to used for transformation (can be a list)
       outFile: save xslt transformation to given file
       xsltProcPath: path to the executable, if None will try to find it
       xsltProcOptions: a string or a list of string of options"""

  RETURN_VAL = {
    0:'normal',
    1:'no argument',
    2:'too many parameters',
    3:'unknown option',
    4:'failed to parse the stylesheet',
    5:'error in the stylesheet',
    6:'error in one of the documents',
    7:'unsupported xsl:output method',
    8:'string parameter contains both quote and double-quotes',
    9:'internal processing error',
    10:'processing was stopped by a terminating message',
    11:'could not write the result to the output file'
  }

  def __init__(self, xslFile=None, xmlFile=None, outFile=None,
               xsltProcPath=None, xsltProcOptions=None):
    t = TraceScope(fsLogger, "XsltProcWrapper.__init__")
    super(XsltProcWrapper, self).__init__()
    self.__installBinDir = globalConfig.getParameter("netborder.Installation.Bin.Directory")
    self.configure(xslFile, xmlFile, outFile=outFile, xsltProcPath=xsltProcPath,
                   xsltProcOptions=xsltProcOptions)


  def configure(self, xslFile, xmlFile, outFile=None, 
               xsltProcPath=None, xsltProcOptions=None):
    """Configure wrapper, xslFile and xsmlFile are required the rest optionnal.
       xmlFile can be either a file name or a list"""
    t = TraceScope(fsLogger, "XsltProcWrapper.configure")
    
    self.__xslFile = xslFile
    self.__xmlFile = xmlFile
    self.__outFile = outFile
    self.__options = xsltProcOptions

    if xsltProcPath is not None:
      self.__xsltProcPath = xsltProcPath
    else:
      self.__xsltProcPath = self.__findXsltProcExec()


  def call(self):
    """Launch Xsltproc command and return the exit code of the executable"""
    t = TraceScope(fsLogger, "XsltProcWrapper.call")
    
    logAssert(fsLogger, os.path.exists(self.__xsltProcPath))
    logAssert(fsLogger, self.__xmlFile is not None)
    logAssert(fsLogger, self.__xslFile is not None)

    xsltProcCmd = [self.__xsltProcPath]

    #Assemble the actual command from the member attributes
    if self.__outFile is not None:
      xsltProcCmd.append("-o")
      xsltProcCmd.append(self.__outFile)
    
    #If __options is a list append all the options to the command.
    if self.__options is not None:
      if isinstance(self.__options, list):
        for option in self.__options:
          xsltProcCmd.extend(option.split(' '))
      else:
        xsltProcCmd.extend(self.__options.split(' '))
    
    xsltProcCmd.append(self.__xslFile)
    
    #If __xmlFile is a list append all the xml file to the command.
    if isinstance(self.__xmlFile, list):
      for xml in self.__xmlFile:
        xsltProcCmd.append(xml)
    else:
      xsltProcCmd.append(self.__xmlFile)

    #Launch the command
    log_debug(fsLogger, "Launching command: %s", " ".join(xsltProcCmd))
    xsltProcProcess = subprocess.Popen( args=xsltProcCmd,
                                        stdin=subprocess.PIPE,
                                        stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE )
    
    stdout, stderr = xsltProcProcess.communicate()
    
    returnCode = xsltProcProcess.returncode
    log_debug(fsLogger, "Command returned: %s", returnCode)

    warning = list()
    if stderr:
      for line in stderr.split('\n'):
        if line == '':
          continue
        m = re.match('WARN:(.*)', line)
        if m:
          warning.append(m.group(1)) 
          continue
        m = re.match('DEBUG:(.*)', line)
        if m:
          log_debug(fsLogger, m.group(1))
          continue
        log_error(fsLogger, line)
    

    return (returnCode, self.RETURN_VAL[returnCode], warning)


  def __findXsltProcExec(self):
    """Try to find Xstlproc utility"""
    
    t = TraceScope(fsLogger, "XsltProcWrapper.__findXsltProcExec")
    returnPath = None
    execExtension = ''
  
    #Check if os is Windows and set executable to end with .exe
    if os.name == 'nt':
      execExtension = '.exe'

    execName = "xsltproc" + execExtension

    if self.__installBinDir is not None:

      xsltPath = os.path.join(self.__installBinDir, execName)

      if os.path.exists(xsltPath):
        log_debug(fsLogger, "Found %s executable at '%s'" % (execName,
                                                             xsltPath))
        returnPath = xsltPath

    
    #If install dir is not set try to find xsltproc in sys.path.
    #Mostly there when ran in development... 
    #netborder.Installation.Bin.Directory should be defined otherwise.
    if returnPath is None:

      for path in sys.path:
        xsltPath = os.path.join(path, execName)
        if os.path.exists(xsltPath):
          log_debug(fsLogger, "Found %s executable at '%s'" % (execName, 
                                                               xsltPath))
          returnPath = xsltPath
          break


    if returnPath is None:
      log_error(fsLogger, "Could not find %s executable" % execName)


    return returnPath
  

###############################################################################
# Runtest
if __name__ == '__main__':

  fsLogger.setLogLevel(LogLevel.debug)
  
  log_info(fsLogger, "Starting XsltProcWrapper runtest")
  success = True
  generatedPyFile = os.path.join('test-output', 'XsltProcTest.py')

  xsltParam = '--stringparam initVersion 3.0.0'
  xsltProc = XsltProcWrapper("smartUpdateConvertersParser.xsl", "test-input/smartUpdateConverters.xml", outFile=generatedPyFile, xsltProcOptions=[xsltParam])
  
  rv, msg, warnings = xsltProc.call()

  if not rv == 0:
    success = False
    log_error(fsLogger, "Return values is not 0")

  if not msg == 'normal':
    success = False
    log_error(fsLogger, "Return values description is not 'normal'")

  if not warnings == []:
    success = False
    log_error(fsLogger, "Warnings message list not empty")


  #Try to import generated python code from xsltproc.
  sys.path.append('test-output')
  try:
    import XsltProcTest
  except Exception ,e:
    success = False
    log_error(fsLogger, "XsltProcTest.py is not valid. Error=%s" % e)
  
  try:
    os.remove(generatedPyFile)
  except OSError, e:
    success = False
    log_error(fsLogger, "Failed to delete file '%s'. Error=%s" % e)

  #Deleted .pyc
  if os.path.exists(generatedPyFile + "c"):
    os.remove(generatedPyFile + "c")

  if success:
    log_info(fsLogger, "Runtest passed")
  else:
    log_error(fsLogger, "Runtest failed")
## !!MIRROR FILE, DO NOT MODIFY!!
