<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:exsl="http://exslt.org/common">    
    
<xsl:output method="xml"/>

<!--
*********************************************************************************************
                        setWanpipe.xsl style sheet
                        ==========================

  Description:  

  Input:  pstn-config.xml from PBX configurator
  Output: pstn-config.xml with consolidated wanpipe ids
  Param:  
        in_boardListFile = File path to netborder-list-boards result (XML format)


*********************************************************************************************
-->
<xsl:param name="in_boardListFile" select="'UNDEF'"/>

<!-- 
    Copy all template 
      Copy element and attribute and propagate parameters to
      descendant templates
-->
<xsl:template match="@*|node()">
  <!-- detected-boards-document param contains the netborder-list-boards XML document -->
  <xsl:param name="detected-boards-document"/>
  <!-- board-keys-nodeset param contains a nodeset of board keys (see below) -->
  <xsl:param name="board-keys-nodeset"/>
  
  <xsl:copy>
    <xsl:apply-templates select="@*|node()">
      <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
      <xsl:with-param name="board-keys-nodeset" select="exsl:node-set($board-keys-nodeset)"/>
    </xsl:apply-templates>
  </xsl:copy>
</xsl:template>

<!-- 
    Boards element matching template
      This template will invoke the wanpipe-resolver copy template for its child elements
-->
<xsl:template match="boards">
  <xsl:param name="detected-boards-document"/>
  <xsl:param name="board-keys-nodeset"/>
  
  <!-- Copy boards element -->
  <xsl:copy>
    <!-- Copy boards attributes -->
    <xsl:apply-templates select="@*">
      <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
      <xsl:with-param name="board-keys-nodeset" select="exsl:node-set($board-keys-nodeset)"/>
    </xsl:apply-templates>
      
    <!-- Invoke copy template that will resolve wanpipe driver 
          Only on first child, wanpipe-resolver template
          will continue with all siblings
    -->
    <xsl:apply-templates select="child::*[1]" mode="wanpipe-resolver">
      <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
      <xsl:with-param name="board-keys-nodeset" select="exsl:node-set($board-keys-nodeset)"/>
    </xsl:apply-templates>
    
  </xsl:copy>

</xsl:template>

<!--
    boards-keys-copy-filter template
      Default, copy all
-->
<xsl:template match="@*|node()" mode="boards-keys-copy-filter">
  <xsl:param name="id-to-remove"/>

  <xsl:copy>
    <xsl:apply-templates select="@*|node()" mode="boards-keys-copy-filter">
      <xsl:with-param name="id-to-remove" select="$id-to-remove"/>
    </xsl:apply-templates>
  </xsl:copy>
</xsl:template>

<!--
    boards-keys-copy-filter template
      filter board by id-to-remove
-->
<xsl:template match="/boards/board" mode="boards-keys-copy-filter">
  <xsl:param name="id-to-remove"/>

<!--
  <xsl:message>boards-keys-copy-filter: id-to-remove=[<xsl:value-of select="$id-to-remove"/>]</xsl:message>
  <xsl:message>boards-keys-copy-filter: current-id=[<xsl:value-of select="@id"/>]</xsl:message>

  <xsl:choose>
    <xsl:when test="number($id-to-remove) = number(@id)">
      <xsl:message>boards-keys-copy-filter: forget id=<xsl:value-of select="@id"/></xsl:message>
    </xsl:when>
    <xsl:otherwise>
      <xsl:message>boards-keys-copy-filter: copy id=<xsl:value-of select="@id"/></xsl:message>
    </xsl:otherwise>
  </xsl:choose>
-->  
  <xsl:if test="number($id-to-remove) != number(@id)">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" mode="boards-keys-copy-filter">
        <xsl:with-param name="id-to-remove" select="$id-to-remove"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:if>
</xsl:template>

<!--
  Default copy template in wanpipe-resolver mode
  for non wanpipe elements in boards element
    This template should not be hit or we have missed a new board type
-->
<xsl:template match="*" mode="wanpipe-resolver">
  <xsl:param name="detected-boards-document"/>
  <xsl:param name="board-keys-nodeset"/>
  <xsl:param name="board-key-id"/>

    <xsl:message exit="yes">ERROR: wanpipe-resolver template invoked for not supported element=<xsl:value-of select="name(.)"/></xsl:message>
  
</xsl:template>

<!--
    wanpipe-resolver template for boards
      board copy template
-->
<xsl:template match="sangomaAnalogBoard|sangomaDigitalBoard" mode="wanpipe-resolver">
  <xsl:param name="detected-boards-document"/>
  <xsl:param name="board-keys-nodeset"/>

  <!-- Create the board key -->
  <xsl:variable name="board-key">
    <xsl:apply-templates select="." mode="build-key"/>
  </xsl:variable>
  
  <!-- Find a matching board in the boardkeylist -->
  <xsl:variable name="board-key-id"
    select="exsl:node-set($board-keys-nodeset)/boards/board[@key=$board-key]/@id[1]"/>
    
  <xsl:message>DEBUG: wanpipe-resolver looking for template board-id=<xsl:value-of select="@ID"/> with board-key=<xsl:value-of select="$board-key"/></xsl:message>
  <xsl:message>DEBUG: wanpipe-resolver matching detected board-id=<xsl:value-of select="$board-key-id"/></xsl:message>

  <!-- Validate that board exists -->
  <xsl:if test="not($detected-boards-document/pstnConfig/boards/child::*[@ID=$board-key-id])">
    <xsl:message terminate="yes">ERROR: Fail to locate board=<xsl:value-of select="$board-key"/> in detected board list</xsl:message>
  </xsl:if>
  
  
  <xsl:copy>
    <!-- Copy all attributes -->
    <xsl:apply-templates select="@*"/>
    
    <!-- locate the board in the detected-boards-document nodeset 
         to extract wanpipe -->
    <xsl:variable name="board"
          select="$detected-boards-document/pstnConfig/boards/child::*[@ID=$board-key-id]"/>
    <!-- Validate interfaces -->

    <!-- Add missing attributes:
          wanpipe
          pciSlot
          pciBus
    -->
    <xsl:if test="$board/@wanpipe">
      <xsl:attribute name="wanpipe">
      <xsl:value-of select="$board/@wanpipe"/>
      </xsl:attribute>
    </xsl:if>    
    
    <xsl:attribute name="pciSlot">
    <xsl:value-of select="$board/@pciSlot"/>
    </xsl:attribute>
    
    <xsl:attribute name="pciBus">
    <xsl:value-of select="$board/@pciBus"/>
    </xsl:attribute>
    
    <!-- Continue copy with childs -->
    <xsl:apply-templates select="node()" mode="wanpipe-resolver">
      <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
      <xsl:with-param name="board-interfaces-nodeset" 
      select="$detected-boards-document/pstnConfig/boards/child::*[@ID=$board-key-id]"/>
    </xsl:apply-templates>
  </xsl:copy>
  
  <!-- Create a new board-keys-nodeset nodeset by copying all elements but the board id used -->
  <xsl:variable name="filtered-board-keys-nodeset">
    <xsl:apply-templates select="exsl:node-set($board-keys-nodeset)" mode="boards-keys-copy-filter">
      <xsl:with-param name="id-to-remove" select="$board-key-id"/>
    </xsl:apply-templates>
  </xsl:variable>
  
  <!-- Continue with the next sibling -->
  <xsl:apply-templates select="following-sibling::*[1]" mode="wanpipe-resolver">
    <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
    <xsl:with-param name="board-keys-nodeset" select="exsl:node-set($filtered-board-keys-nodeset)"/>
  </xsl:apply-templates>

</xsl:template>

<!--
    wanpipe-resolver template for analog board interfaces
      copy all template and replace wanpipeInterface
-->
<xsl:template match="sangomaAnalogBoard/interface" mode="wanpipe-resolver">
  <xsl:param name="detected-boards-document"/>
  <xsl:param name="board-interfaces-nodeset"/>
  
  <xsl:copy>
    <!-- Copy all attributes -->
    <xsl:apply-templates select="@*[not(@wanpipeInterface)]"/>
    <!-- locate the board in the detected-boards-document nodeset 
         to extract wanpipe -->
    <xsl:variable name="interface-pos" select="count(./preceding-sibling::interface)+1"/>
         
    <xsl:if test="$board-interfaces-nodeset/interface[$interface-pos]/@wanpipeInterface">
      <xsl:attribute name="wanpipeInterface">
        <xsl:value-of select="$board-interfaces-nodeset/interface[$interface-pos]/@wanpipeInterface"/>
      </xsl:attribute>
    </xsl:if>
    
    <xsl:apply-templates select="node()">
      <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
    </xsl:apply-templates>
  </xsl:copy>

</xsl:template>

<!--
    wanpipe-resolver template for digital board interfaces
      copy all template and replace wanpipe
-->
<xsl:template match="sangomaDigitalBoard/interface" mode="wanpipe-resolver">
  <xsl:param name="detected-boards-document"/>
  <xsl:param name="board-interfaces-nodeset"/>
  
  <xsl:copy>
    <!-- Copy all attributes -->
    <xsl:apply-templates select="@*[not(@wanpipe)]"/>
    <!-- locate the board in the detected-boards-document nodeset 
         to extract wanpipe -->
    <xsl:variable name="interface-pos" select="count(./preceding-sibling::interface)+1"/>
         
    <xsl:if test="$board-interfaces-nodeset/interface[$interface-pos]/@wanpipe">
      <xsl:attribute name="wanpipe">
        <xsl:value-of select="$board-interfaces-nodeset/interface[$interface-pos]/@wanpipe"/>
      </xsl:attribute>
    </xsl:if>
    
    <xsl:apply-templates select="node()">
      <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
    </xsl:apply-templates>
  </xsl:copy>

</xsl:template>

<!-- 
  Create a board key for an analog board
    Includes:
      sangomaAnalogBoard
      <model>
      and concat all interface type together
    For example, the following detected board:
      <sangomaAnalogBoard ID="1" displayName="B2 - A200_analog" model="A200" pciSlot="2" pciBus="9" wanpipe="127">
        <interface ID="1" displayName="B1I1" type="FXS" phoneNumber="" callerName="" wanpipeInterface="100"/>
        <interface ID="2" displayName="B1I2" type="FXS" phoneNumber="" callerName="" wanpipeInterface="200"/>
        <interface ID="3" displayName="B1I3" type="FXO" phoneNumber="" wanpipeInterface="300"/>
        <interface ID="4" displayName="B1I4" type="FXO" phoneNumber="" wanpipeInterface="400"/>
      </sangomaAnalogBoard>
      
    Will create the key: sangomaAnalogBoardA200FXSFXSFXOFXO
    
-->
<xsl:template match="sangomaAnalogBoard" mode="build-key">
  <xsl:variable name="interfaces-key">
    <xsl:for-each select="interface">
      <xsl:value-of select="@type"/>
    </xsl:for-each>
  </xsl:variable>    
  <xsl:value-of select="concat(name(.),@model,$interfaces-key)"/>
  
</xsl:template>

<xsl:template match="sangomaDigitalBoard" mode="build-key">
  <xsl:variable name="interfaces-key">
    <xsl:for-each select="interface">
      <xsl:value-of select="@briModType"/>
    </xsl:for-each>
  </xsl:variable>    
  <xsl:value-of select="concat(name(.),@model,$interfaces-key)"/>
  
</xsl:template>

<!-- 
  Root element 
-->
<xsl:template match="/">
  <!-- Read the result of netborder-list-boards -->
  <xsl:variable name="detected-boards-document" select="document($in_boardListFile)"/>
  
  <!--
    Create the board keys nodeset. Used to find a detected board by its keys,
    cross search between template and netborder-list-boards result
    
    Example:

    netborder-list-boards result:
    <pstnConfig version="3.1">
      <boards>
        <sangomaDigitalBoard ID="1" displayName="B1 - A102_digital" model="A102" pciSlot="4" pciBus="6">
          <interface ID="1" displayName="B1I1" wanpipe="6"/>
          <interface ID="2" displayName="B1I2" wanpipe="7"/>
        </sangomaDigitalBoard>
        <sangomaDigitalBoard ID="3" displayName="B3 - A500_digital" model="A500" pciSlot="4" pciBus="8">
          <interface ID="1" displayName="B3I1" wanpipe="8" briModType="NETWORK"/>
          <interface ID="2" displayName="B3I2" wanpipe="9" briModType="NETWORK"/>
          <interface ID="3" displayName="B3I3" wanpipe="10" briModType="TERMINAL"/>
          <interface ID="4" displayName="B3I4" wanpipe="11" briModType="TERMINAL"/>
        </sangomaDigitalBoard>
        <sangomaAnalogBoard ID="2" displayName="B2 - A200_analog" model="A200" pciSlot="2" pciBus="9" wanpipe="1">
          <interface ID="1" displayName="B2I1" type="FXS" phoneNumber="" callerName="" wanpipeInterface="1"/>
          <interface ID="2" displayName="B2I2" type="FXS" phoneNumber="" callerName="" wanpipeInterface="2"/>
          <interface ID="3" displayName="B2I3" type="FXO" phoneNumber="" wanpipeInterface="3"/>
          <interface ID="4" displayName="B2I4" type="FXO" phoneNumber="" wanpipeInterface="4"/>
        </sangomaAnalogBoard>
      </boards>
    </pstnConfig>      
    
    Will create the following nodeset  
    <boards>
      <board key="sangomaDigitalBoardA102" id="1"/>
      <board key="sangomaDigitalBoardA500NETWORKNETWORKTERMINALTERMINAL" id="3"/>
      <board key="sangomaAnalogBoardA200FXSFXSFXOFXO" id="2"/>
    </boards>
    
  -->
  <xsl:variable name="board-keys-nodeset">
    <xsl:element name="boards">
      <xsl:for-each select="$detected-boards-document/pstnConfig/boards/child::*">
        <xsl:element name="board">
          <xsl:attribute name="key">
            <xsl:apply-templates select="." mode="build-key"/>
          </xsl:attribute>
          <xsl:attribute name="id">
            <xsl:value-of select="@ID"/>
          </xsl:attribute> 
          <xsl:attribute name="pciSlot">
            <xsl:value-of select="@pciSlot"/>
          </xsl:attribute> 
          <xsl:attribute name="pciBus">
            <xsl:value-of select="@pciBus"/>
          </xsl:attribute> 
        </xsl:element>
        <!-- Debug -->
        <xsl:message>DEBUG: board-keys add detected board-id=<xsl:value-of select="@ID"/> board-key=<xsl:apply-templates select="." mode="build-key"/></xsl:message>
      </xsl:for-each>
    </xsl:element>
  </xsl:variable>
  
  <!-- Start copying the complete pstn config template -->
  <xsl:apply-templates select="@*|node()">
    <xsl:with-param name="detected-boards-document" select="$detected-boards-document"/>
    <xsl:with-param name="board-keys-nodeset" select="exsl:node-set($board-keys-nodeset)"/>
  </xsl:apply-templates>
  
</xsl:template>

</xsl:stylesheet> 




















